@@ -26,11 +26,11 @@ from requests import HTTPError
 
 from manifesttool.dev_tool import defaults
 from manifesttool.dev_tool.actions.create import bump_minor
+from manifesttool.dev_tool.actions.create \
+    import register_parser as register_create_parser
 from manifesttool.dev_tool.actions.create import create_dev_manifest
 from manifesttool.dev_tool.pelion import pelion
-from manifesttool.mtool.actions import existing_file_path_arg_factory
 from manifesttool.mtool.actions import non_negative_int_arg_factory
-from manifesttool.mtool.actions import semantic_version_arg_factory
 from manifesttool.mtool.asn1 import ManifestAsnCodecBase
 
 logger = logging.getLogger('manifest-dev-tool-update')
@@ -41,84 +41,35 @@ MAX_NAME_LEN = 128
 
 def register_parser(parser: argparse.ArgumentParser, schema_version: str):
 
-    required = parser.add_argument_group('required arguments')
-    optional = parser.add_argument_group('optional arguments')
+    register_create_parser(parser, schema_version, True)
 
-    required.add_argument(
-        '-p', '--payload-path',
-        help='Payload local path - for digest calculation.',
-        type=existing_file_path_arg_factory,
-        required=True
+    pdm_group = parser.add_argument_group(
+        'optional arguments (Tool\'s campaign management)'
     )
 
-    if schema_version == 'v1':
-        optional.add_argument(
-            '-v', '--fw-version',
-            type=non_negative_int_arg_factory,
-            help='FW version to be set in manifest. '
-                 '[Default: current timestamp]',
-            default=int(time.time())
-        )
-    else:
-        optional.add_argument(
-            '-v', '--fw-version',
-            type=semantic_version_arg_factory,
-            help='FW version to be set in manifest in Semantic '
-                 'Versioning Specification format.'
-        )
-        optional.add_argument(
-            '--component-name',
-            default='MAIN',
-            metavar='NAME',
-            help='Component name to be updated. Must correspond to existing '
-                 'components name on targeted devices'
-        )
-        optional.add_argument(
-            '-m', '--sign-image',
-            action='store_true',
-            help='Sign image. Should be used when bootloader on a device '
-                 'expects signed FW image.'
-        )
-
-    optional.add_argument(
-        '-r', '--priority',
-        type=non_negative_int_arg_factory,
-        help='Update priority >=0. [Default: 0]',
-        metavar='INT',
-        default=0
-    )
-
-    optional.add_argument(
-        '-d', '--vendor-data',
-        help='Vendor custom data - to be passed to a device.',
-        type=existing_file_path_arg_factory
-    )
-
-    pdm_group = parser.add_argument_group('optional PDM portal arguments')
-
     pdm_group.add_argument(
         '-i', '--device-id',
-        help='Device ID for for targeting a specific device in '
+        help='Device ID for targeting a specific device in '
              'update campaign filter.'
     )
 
     pdm_group.add_argument(
         '-s', '--start-campaign',
         action='store_true',
-        help='Start update campaign automatically.'
+        help='Create and start update campaign automatically.'
     )
     pdm_group.add_argument(
         '-w', '--wait-for-completion',
         dest='wait',
         action='store_true',
-        help='Start update campaign automatically, '
-              'wait for it to finish and cleanup created resources.'
+        help='Create and start update campaign automatically, '
+             'wait for it to finish, and clean up created resources.'
     )
     pdm_group.add_argument(
         '-t', '--timeout',
         type=non_negative_int_arg_factory,
         help='Wait timeout in seconds. '
-             'Only relevant in case --wait-for-completion was provided. '
+             'Only relevant if --wait-for-completion was provided. '
              '[Default: 0 - wait-forever]',
         default=0
     )
@@ -126,22 +77,7 @@ def register_parser(parser: argparse.ArgumentParser, schema_version: str):
         '-n', '--no-cleanup',
         action='store_true',
         help='Skip cleaning up created resources. '
-             'Only relevant in case --wait-for-completion was provided.'
-    )
-
-    optional.add_argument(
-        '--cache-dir',
-        help='Tool cache directory. '
-             'Must match the directory used by "init" command',
-        type=Path,
-        default=defaults.BASE_PATH
-    )
-
-    optional.add_argument(
-        '-h',
-        '--help',
-        action='help',
-        help='show this help message and exit'
+             'Only relevant if --wait-for-completion was provided.'
     )
 
 
@@ -160,8 +96,6 @@ def _wait(api: pelion.UpdateServiceApi, campaign_id: pelion.ID, timeout):
             if not api.campaign_is_active(curr_phase):
                 logger.info(
                     "Campaign is finished in state: %s", curr_state)
-                devices_updated = api.assert_all_device_updated(campaign_id)
-                logger.info("%d devices successfully updated", devices_updated)
                 return
             time.sleep(1)
         logger.error('Campaign timed out')
@@ -170,6 +104,63 @@ def _wait(api: pelion.UpdateServiceApi, campaign_id: pelion.ID, timeout):
         logger.error('Failed to retrieve campaign state')
         raise
 
+def _print_summary(summary: dict):
+    logger.info("----------------------------")
+    logger.info("    Campaign Summary ")
+    logger.info("----------------------------")
+    logger.info(" Successfully updated:   %d",
+                summary.get('success') if summary.get('success') else 0)
+    logger.info(" Failed to update:       %d",
+                summary.get('fail') if summary.get('fail') else 0)
+    logger.info(" Skipped:                %d",
+                summary.get('skipped') if summary.get('skipped') else 0)
+    logger.info(" Pending:                %d",
+                summary.get('info') if summary.get('info') else 0)
+    logger.info(" Total in this campaign: %d", sum(summary.values()))
+
+
+def _finalize(api: pelion.UpdateServiceApi,
+              do_cleanup: bool,
+              campaign_id: pelion.ID,
+              manifest_id: pelion.ID,
+              fw_image_id: pelion.ID,
+              manifest_path: Path):
+    summary = []
+    failed_devices = []
+    try:
+        if campaign_id:
+            # stop the campaign if it's still active
+            api.campaign_stop(campaign_id)
+            # get summary and failed devices
+            statistics = api.campaign_statistics(campaign_id)
+            summary = {s['id']: s['count'] for s in statistics}
+            if summary.get('fail') and summary.get('fail') > 0:
+                devices_state = api.campaign_device_metadata(campaign_id)
+                failed_devices = [d['device_id'] for d in devices_state
+                                  if d['deployment_state'] != 'deployed']
+        if do_cleanup:
+            logger.info('Cleaning up resources...')
+            if campaign_id:
+                api.campaign_delete(campaign_id)
+            if manifest_id:
+                api.manifest_delete(manifest_id)
+            if fw_image_id:
+                api.fw_delete(fw_image_id)
+            if manifest_path and manifest_path.is_file():
+                manifest_path.unlink()
+    except HTTPError as ex:
+        logger.error('Failed to finalize update campaign')
+        logger.debug('Exception %s', ex, exc_info=True)
+
+    # print campaign summary
+    if summary:
+        _print_summary(summary)
+        if failed_devices:
+            # assert if not all devices were updated
+            raise AssertionError(
+                'Failed to update {} devices: {}'.format(
+                    len(failed_devices), ', '.join(failed_devices)))
+
 
 def update(
         payload_path: Path,
@@ -181,11 +172,12 @@ def update(
         do_wait: bool,
         do_start: bool,
         timeout: int,
-        skip_cleanup: bool,
+        do_cleanup: bool,
         service_config: Path,
         fw_version: str,
         sign_image: bool,
-        component: str
+        component: str,
+        short_url: bool
 ):
     config = load_service_config(service_config)
 
@@ -199,7 +191,7 @@ def update(
         timestamp = time.strftime('%Y_%m_%d-%H_%M_%S')
         logger.info('Uploading FW image %s', payload_path.as_posix())
 
-        fw_image_url, fw_image_id = api.fw_upload(
+        fw_image_url, short_image_url, fw_image_id = api.fw_upload(
             fw_name='{timestamp}-{filename}'.format(
                 filename=payload_path.name,
                 timestamp=timestamp),
@@ -211,7 +203,7 @@ def update(
             manifest_version=manifest_version,
             vendor_data_path=vendor_data,
             payload_path=payload_path,
-            payload_url=fw_image_url,
+            payload_url=short_image_url if short_url else fw_image_url,
             priority=priority,
             fw_version=fw_version,
             sign_image=sign_image,
@@ -255,32 +247,24 @@ def update(
     except KeyboardInterrupt:
         logger.error('Aborted by user...')
     finally:
-        if not skip_cleanup and do_wait:
-            try:
-                logger.info('Cleaning up resources.')
-                if campaign_id:
-                    api.campaign_stop(campaign_id)
-                    api.campaign_delete(campaign_id)
-                if manifest_id:
-                    api.manifest_delete(manifest_id)
-                if fw_image_id:
-                    api.fw_delete(fw_image_id)
-                if manifest_path and manifest_path.is_file():
-                    manifest_path.unlink()
-            except HTTPError as ex:
-                logger.error('Failed to cleanup resources')
-                logger.debug('Cleanup exception %s', ex, exc_info=True)
+        if do_wait:
+            _finalize(api,
+                      do_cleanup,
+                      campaign_id,
+                      manifest_id,
+                      fw_image_id,
+                      manifest_path)
 
 
 def load_service_config(service_config):
     if service_config.is_file():
         with service_config.open('rt') as fh:
             config = yaml.safe_load(fh)
-    else:
-        raise AssertionError('Pelion service configurations (URL and API key) '
-                             'are not provided for assisted campaign '
-                             'management')
-    return config
+            if 'host' in config and 'api_key' in config:
+                return config
+    raise AssertionError('Pelion service configurations (URL and access key) '
+                         'are not provided for assisted campaign '
+                         'management')
 
 
 def entry_point(
@@ -328,9 +312,10 @@ def entry_point(
         do_start=args.start_campaign or args.wait,
         do_wait=args.wait,
         timeout=args.timeout,
-        skip_cleanup=args.no_cleanup,
+        do_cleanup=not args.no_cleanup,
         service_config=cache_dir / defaults.CLOUD_CFG,
         fw_version=fw_version,
         sign_image=getattr(args, 'sign_image', False),
-        component=component_name
+        component=component_name,
+        short_url=hasattr(args, 'use_short_url') and args.use_short_url
     )
